#define RK 1
#define __RK__ 1

#include <hourglass.h>
#include <linux/include/rk/rk.h>
#include <signal.h>

#define MAX_SETS 100

rk_resource_set_t *rs = NULL;

void make_res (unsigned int amount, 
	       unsigned int period,
	       int soft,
	       int which)
{
  rk_reserve_t cpu_rsv;
  struct timespec c, t;

  if (!rs) {
    rs = (rk_resource_set_t *) malloc (MAX_SETS * sizeof (rk_resource_set_t));
    if (!rs) {
      printf ("oops: couldn't allocate resource set array\n");
      exit (-1);
    }
    memset (rs, 0, MAX_SETS * sizeof (rk_resource_set_t));
  }

  if (!rs[which]) {
    rs[which] = rk_resource_set_create();
    if (!rs[which]) {
      printf ("oops: could not create resource set\n");
      exit (-1);
    }
  }

  {
    rk_reserve_param_data_t p;
    c.tv_sec = amount/1000000, c.tv_nsec = (amount%1000000)*1000;
    t.tv_sec = period/1000000, t.tv_nsec = (period%1000000)*1000;
    printf ("thread %d res: amount sec = %ld, nsec = %ld\n", 
	    which, c.tv_sec, c.tv_nsec);
    printf ("thread %d res: period sec = %ld, nsec = %ld\n", 
	    which, t.tv_sec, t.tv_nsec);
    if (soft) {
      p.enf_mode = RSV_SOFT;
      p.sch_mode = RSV_SOFT;
      p.rep_mode = RSV_SOFT;
    } else {
      p.enf_mode = RSV_HARD;
      p.sch_mode = RSV_HARD;
      p.rep_mode = RSV_HARD;
    }
    cpu_rsv = rk_cpu_reserve_create(rs[which], &c, &t, &p);
  }

  if (!cpu_rsv) {
    printf ("oops: could not create CPU reservation\n");
    exit (-1);
  }

  /* attach the current process to the resource set */
  rk_resource_set_attach_process(rs[which], getpid());  
}

void delete_res (int which)
{
  if (rs[which]) {
    rk_resource_set_destroy (rs[which]);
  }
}
