/*
 *
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "hourglass.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sched.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <errno.h>
#include <signal.h>
#include <math.h>
#include <posix_time.h>

/*
 * timers based on high-resolution Linux timers
 *
 * see: http://sourceforge.net/projects/high-res-timers/
 *
 */

static void hr_timed_sleep (int id, us_time t)
{
  int flags;
  struct timespec waket;

  if (t < 0) {
    t = -t;
    flags = 0;
  } else {
    flags = TIMER_ABSTIME;
  }

  waket.tv_sec = (int)t / 1000000;
  waket.tv_nsec = 1000 * (((long long int)t) % 1000000);

  clock_nanosleep (CLOCK_REALTIME_HR, flags, &waket, NULL);
}

static void hr_init_timers (void)
{
}

static void hr_deinit_timers (void)
{
}

const struct timer_funcs hr_timer_funcs = {
  hr_timed_sleep,
  hr_init_timers,
  hr_deinit_timers
};
