/*
 *
 * Copyright (c) 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

/*
 * much code in this file was grabbed from the MSDN Kibrary -- see the
 * entries under "multimedia timers" 
 *
 * remaining code was copied from rtc_timers.c in Hourglass; it seems
 * to be okay to mix win32 and posix thread calls (yikes)
 */

#include "hourglass.h"
#include "windows.h"

struct mm_timer {
  int set;
  us_time expire;
  pthread_mutex_t mut;
  pthread_cond_t cond;
};

static struct mm_timer *mm_timers;
static int MM_TIMER_INIT = 0;
static MMRESULT TimerId;
static UINT wTimerRes;

static void mm_timed_sleep (int id, us_time t)
{
  ASSERT (MM_TIMER_INIT);

  pthread_mutex_lock (&mm_timers[id].mut);
  ASSERT (!mm_timers[id].set);
  mm_timers[id].set = 1;

  if (t < 0) {
    // negative time is relative sleep
    us_time now = cycle_to_us (rdtsc());
    mm_timers[id].expire = now - t;
  } else {
    // positive time is absolute
    mm_timers[id].expire = t;
  }

  ASSERT (mm_timers[id].set);

  pthread_cond_wait (&mm_timers[id].cond,
		     &mm_timers[id].mut);

  ASSERT (!mm_timers[id].set);

  pthread_mutex_unlock (&mm_timers[id].mut);
}

static void *timer_thrd_func (void *arg)
{
  return NULL;
}

static void CALLBACK callback (UINT a, UINT b, DWORD c, DWORD d, DWORD e)
{
  int i;
  us_time now = cycle_to_us (rdtsc ());
  
  for (i=0; i<num_threads; i++) {
    if (mm_timers[i].set && 
	now >= mm_timers[i].expire) {
      pthread_mutex_lock (&mm_timers[i].mut);
      mm_timers[i].set = 0;
      pthread_cond_signal (&mm_timers[i].cond);
      pthread_mutex_unlock (&mm_timers[i].mut);
    }
  }
}

static void mm_init_timers (void)
{
  MMRESULT res;
  TIMECAPS caps;
  int i;

  res = timeGetDevCaps (&caps, sizeof (TIMECAPS));
  if (res != TIMERR_NOERROR) {
    printf ("oops: problem with timeGetDevCaps\n");
    exit (-1);
  }

  // set to highest possible resolution -- usually 1ms
  wTimerRes = caps.wPeriodMin;

  printf ("timer resolution will be %d ms\n", wTimerRes);

  res = timeBeginPeriod (wTimerRes); 
  if (res != TIMERR_NOERROR) {
    printf ("oops: problem with timeBeginPeriod\n");
    exit (-1);
  }  

  mm_timers = (struct mm_timer *) 
    xmalloc (num_threads * sizeof (struct mm_timer));

  for (i=0; i<num_threads; i++) {
    mm_timers[i].set = 0;
    pthread_mutex_init (&mm_timers[i].mut, NULL);
    pthread_cond_init (&mm_timers[i].cond, NULL);
  }

  TimerId = timeSetEvent (1, 1,
			  &callback,
			  0,
			  TIME_PERIODIC);
  if (!TimerId) {
    printf ("oops: problem with timeSetEvent\n");
    exit (-1);
  }

  MM_TIMER_INIT = 1;
}

static void mm_deinit_timers (void)
{
  timeKillEvent (TimerId);
  timeEndPeriod (wTimerRes);
  MM_TIMER_INIT = 0;
}

const struct timer_funcs mm_timer_funcs = {
  mm_timed_sleep,
  mm_init_timers,
  mm_deinit_timers
};
