/*
 *
 * Copyright (c) 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "hourglass.h"

static void make_sched_param (enum thrd_pri_t pri, 
			      int *policy,
			      struct sched_param *param,
			      int *niceval)
{
  int max_realtime_pri;

  max_realtime_pri = sched_get_priority_max (SCHED_RR);
  if (max_realtime_pri == -1) {
    perror ("sched_get_priority_max");
    exit (errno);
  }

  DBGPrint (8, ("max_realtime_priority == %d\n", max_realtime_pri));

  switch (pri) {

  case IDLE:
    param->sched_priority = 0;
    *policy = SCHED_OTHER;
    *niceval = 19;
    break;

  case LOW:
    param->sched_priority = 0;
    *policy = SCHED_OTHER;
    *niceval = 10;
    break;

  case NORMAL:
    param->sched_priority = 0;
    *policy = SCHED_OTHER;
    *niceval = 0;
    break;
    
  case HIGH:
    param->sched_priority = 0;
    *policy = SCHED_OTHER;
    *niceval = -10;
    break;
    
  case HIGHEST:
    param->sched_priority = 0;
    *policy = SCHED_OTHER;
    *niceval = -19;
    break;
    
  case RTLOW:
    param->sched_priority = max_realtime_pri / 4;
    *policy = SCHED_RR;
    break;
    
  case RTMED:
    param->sched_priority = max_realtime_pri / 2;
    *policy = SCHED_RR;
    break;
    
  case RTHIGH:
    param->sched_priority = max_realtime_pri - 2;
    *policy = SCHED_RR;
    break;
    
  case TIMER_PRI:
    param->sched_priority = max_realtime_pri - 1;
    *policy = SCHED_RR;
    break;

  case MAIN_PRI:
    param->sched_priority = max_realtime_pri;
    *policy = SCHED_RR;
    break;

  default:
    printf ("Hrm: unexpected arg to set_priority: %d\n", pri);
    exit (-1);
  }

  DBGPrint (5, ("sched_priority = %d\n", param->sched_priority));
  DBGPrint (5, ("policy = %d\n", *policy));
}

void set_priority (enum thrd_pri_t pri)
{
  struct sched_param param;
  int policy, res, niceval;

  make_sched_param (pri, &policy, &param, &niceval);

  res = sched_setscheduler (0, policy, &param);
  if (res != 0) {
    perror ("sched_setscheduler");
    printf ("You probably need to be running as root.\n");
    exit (errno);
  }

  if (policy == SCHED_OTHER) {
    errno = 0;
    res = setpriority (PRIO_PROCESS, 0, niceval);
    if (errno != 0) {
      perror ("setpriority");
      exit (errno);
    }
  }
}

#define MLOCK_ERR_1a "    **** WARNING: lack permissions to lock pages ****\n"
#define MLOCK_ERR_1b "    **** WARNING: mlockall() not supported on this platform ****\n"
#define MLOCK_ERR_2  "    **** results from this run may be unreliable ****\n"

void lock_pages (void)
{
#ifdef HAVE_MLOCKALL
  int res;
  
  res = mlockall (MCL_CURRENT | MCL_FUTURE);
  if (res != 0) {
    perror ("mlockall");
    printf ("%s%s", MLOCK_ERR_1a, MLOCK_ERR_2);
  }
#else
  printf ("%s%s", MLOCK_ERR_1b, MLOCK_ERR_2);
#endif
}

