/*
 *
 * Copyright (c) 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "hourglass.h"

// ugh -- can't seem to get accurate MHz on win2k
#ifdef WIN32
#define MHZ_SLOP 5
#else
#define MHZ_SLOP 2
#endif

void *xmalloc (size_t size)
{
  void *v = malloc (size);
  if (!v) {
    printf ("whoops, malloc failure -- exiting\n");
    exit (-1);
  }
  return v;
}

static us_time get_timeval_us (void)
{
  struct timeval tv;
  us_time t;
  
  gettimeofday (&tv, NULL);
  t = tv.tv_usec + (1000000 * tv.tv_sec);
  return t;
}

static double my_fabs (double d)
{
  if (d < 0) {
    return -d;
  } else {
    return d;
  }
}

double find_mhz (void)
{
  double last_freq, freq = 0;
  int i;
  int good = 0;
  us_time us_start, us_stop, us_diff;
  cycle_time cycle_start, cycle_stop, cycle_diff;
  int saved_generate_gap_hist = generate_gap_hist;

  generate_gap_hist = 0;

  set_priority (MAIN_PRI);

  for (i=0; i<100; i++) {

    // estimate MHz

    us_start = get_timeval_us ();
    cycle_start = rdtsc ();

#if 0
    /*
     * ugh -- sleeping permits some laptops to power down their
     * processors, which screws up the measurements -- so busy-wait
     * instead
     */
    usleep (1000);
#else
    {
      volatile int x;
      for (x=0; x < 5*1000*1000; x++) {
      }
    }
#endif

    us_stop = get_timeval_us ();
    cycle_stop = rdtsc ();

    us_diff = us_stop - us_start;
    cycle_diff = cycle_stop - cycle_start;

    last_freq = freq;
    freq = cycle_diff / us_diff;

    DBGPrint (3, ("freq = %f\n", freq));

    // wait for three successive values within range

    if (my_fabs (last_freq - freq) < MHZ_SLOP) {
      good++;
      if (good == 3) {
	set_priority (NORMAL);
	generate_gap_hist = saved_generate_gap_hist;
	return freq;
      }
    } else {
      good = 0;
    }
  }
  
  set_priority (NORMAL);

  printf ("couldn't determine MHz of this processor: try reducing\n");
  printf ("machine load or increasing the value of MHZ_SLOP in\n");
  printf ("utils.c\n");

  exit (-1);
}
