/*
 *
 * Copyright (c) 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "hourglass.h"
#include "windows.h"

static void make_sched_param (enum thrd_pri_t pri, 
			      int *class,
			      int *priority)
{
  switch (pri) {

  case IDLE:
    *priority = THREAD_PRIORITY_IDLE;
    *class = NORMAL_PRIORITY_CLASS;
    break;

  case LOW:
    *priority = THREAD_PRIORITY_BELOW_NORMAL;
    *class = NORMAL_PRIORITY_CLASS;
    break;

  case NORMAL:
    *priority = THREAD_PRIORITY_NORMAL;
    *class = NORMAL_PRIORITY_CLASS;
    break;
    
  case HIGH:
    *priority = THREAD_PRIORITY_ABOVE_NORMAL;
    *class = NORMAL_PRIORITY_CLASS;
    break;
    
  case HIGHEST:
    *priority = THREAD_PRIORITY_HIGHEST;
    *class = NORMAL_PRIORITY_CLASS;
    break;
    
  case RTLOW:
    *priority = THREAD_PRIORITY_BELOW_NORMAL;
    *class = REALTIME_PRIORITY_CLASS;
    break;
    
  case RTMED:
    *priority = THREAD_PRIORITY_NORMAL;
    *class = REALTIME_PRIORITY_CLASS;
    break;
    
  case RTHIGH:
    *priority = THREAD_PRIORITY_ABOVE_NORMAL;
    *class = REALTIME_PRIORITY_CLASS;
    break;
    
  case TIMER_PRI:
    *priority = THREAD_PRIORITY_HIGHEST;
    *class = REALTIME_PRIORITY_CLASS;
    break;

  case MAIN_PRI:
    *priority = THREAD_PRIORITY_TIME_CRITICAL;
    *class = REALTIME_PRIORITY_CLASS;
    break;

  default:
    printf ("Hrm: unexpected arg to set_priority: %d\n", pri);
    exit (-1);
  }
}

void set_priority (enum thrd_pri_t pri)
{
  unsigned int class;
  int priority;
  HANDLE proc, thr, tok;
  LUID luid;
  TOKEN_PRIVILEGES tp;
  BOOL rv;

  make_sched_param (pri, &class, &priority);

  thr = GetCurrentThread ();
  proc = GetCurrentProcess ();

  rv = OpenProcessToken (proc, TOKEN_ADJUST_PRIVILEGES, &tok);
  if (!rv) {
    printf ("oops - OpenProcessToken failed!\n");
    exit (-1);
  }

 if (!LookupPrivilegeValue(NULL,
                            "SeIncreaseBasePriorityPrivilege",
                            &luid)) {
    printf("LookupPrivilegeValue error: %lu\n", GetLastError() );
    exit (-1);
  }

  tp.PrivilegeCount = 1;
  tp.Privileges[0].Luid = luid;
  tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
  
  AdjustTokenPrivileges(tok,
                        FALSE,
                        &tp,
                        sizeof(TOKEN_PRIVILEGES),
                        (PTOKEN_PRIVILEGES) NULL,
                        (PDWORD) NULL);
  
  if (GetLastError() != ERROR_SUCCESS) {
    printf("AdjustTokenPrivileges failed: %lu\n", GetLastError());
    exit (-1);
  }
  
  if (SetPriorityClass (proc, class) == FALSE) {
    printf ("SetPriorityClass failed\n");
    exit (-1);
  }

  if (SetThreadPriority (thr, priority) == FALSE) {
    printf ("SetThreadPriority failed\n");
    exit (-1);
  }
}

#define MLOCK_ERR_1a "    **** WARNING: lack permissions to lock pages ****\n"
#define MLOCK_ERR_1b "    **** WARNING: mlockall() not supported on this platform ****\n"
#define MLOCK_ERR_2  "    **** results from this run may be unreliable ****\n"

void lock_pages (void)
{
#ifdef HAVE_MLOCKALL
  int res;
  
  res = mlockall (MCL_CURRENT | MCL_FUTURE);
  if (res != 0) {
    perror ("mlockall");
    printf ("%s%s", MLOCK_ERR_1a, MLOCK_ERR_2);
  }
#else
  printf ("%s%s", MLOCK_ERR_1b, MLOCK_ERR_2);
#endif
}

